%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% bl_type.m                                                              %
%                                                                        %
% This program reads in data the processed sonic anemometer data and raw %
% Doppler lidar data for the of dates and determines the probability of  %
% each of the nine boundary-layer types in Harvey at al. (2013).         %
%                                                                        %
% Copyright (c) 2012-2013 Natalie Harvey and the University of Reading   %
%                                                                        %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Specify dates
date0 = 20110730;  % This is the day before the first element of dates
dates = [20110731,20110801:20110803];

% Specify Doppler lidar data location
ddir = ['./data/'];

% Specify directory containing output from pst.m
pstdir = ['./pst/'];

% Read in phi_st from pst file
pstfile = ['./pst/pst_20110730_20110803.txt'];
load(pstfile)
phi_st_full = pst_20110730_20110803(:,2:25);
phi_st_dates = pst_20110730_20110803(:,1);

% Specify output directory
odir = ['./output/'];

% Set threshold values
var_thres = 0.1;
back_thres = 5e-5;
frac_clear_thres = 0.95;

% Threshold fraction of data required for calculating variance and skewness
fraction_good_data_reqd = 0.5;
  
% Temporal resolution in hours for cloudbase height and aerosol layer depth
dtime = 1;
% Temporal resolution in hours used to calculate skewness and turbulence
dtime_sk = 2;

% Read in H and H_err from pst file (for plotting only)
Hfile = ['./pst/H_20110730_20110803.txt'];
H_errfile = ['./pst/H_err_20110730_20110803.txt'];
load(Hfile)
load(H_errfile)
H_full = H_20110730_20110803(:,2:25);
H_err_full = H_err_20110730_20110803(:,2:25);
H_dates = H_20110730_20110803(:,1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Initialise BL Type field
BL_TYPE = zeros(length(dates),24);
P_BL_1 = zeros(length(dates),24);
P_BL_2 = zeros(length(dates),24);
P_BL_3 = zeros(length(dates),24);
P_BL_4 = zeros(length(dates),24);
P_BL_5 = zeros(length(dates),24);
P_BL_6 = zeros(length(dates),24);
P_BL_7 = zeros(length(dates),24);
P_BL_8 = zeros(length(dates),24);
P_BL_9 = zeros(length(dates),24);

% Loop through each date
for id = 1:length(dates);

  % Check sonic data is there and is finite
  sonic_ind(id) = find(phi_st_dates == dates(id));
  if isfinite(sonic_ind(id))
  sonic_good = isfinite(nansum(phi_st_full(sonic_ind(id),:)));
  if sonic_good == 1
  
  phi_st(id,:) = phi_st_full(sonic_ind(id),:);
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Read in the lidar data, and combine into single-day periods %
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  % Clear the "d" structure where the data will be put
  clear d
  % Convert number to a string
  if id == 1
    day_before = num2str(date0);
  else
    day_before = num2str(dates(id-1));
  end
  if ~exist('d');
    % Load the day before lidar data
    infile=[ddir day_before '_chilbolton_halo-doppler-lidar.nc'];
    [d,a] = load_nc_struct(infile);
    if isempty(d)
      warning(['Doppler lidar data from ' day_before ' not found']);
      BL_TYPE(id,1) = 9999; %if day before data is not present
      P_BL_1(id,1) = 9999;
      P_BL_2(id,1) = 9999;
      P_BL_3(id,1) = 9999;
      P_BL_4(id,1) = 9999;
      P_BL_5(id,1) = 9999;
      P_BL_6(id,1) = 9999; 
      P_BL_7(id,1) = 9999; 
      P_BL_8(id,1) = 9999; 
      P_BL_9(id,1) = 9999;
    end    
  end 
    
  % Clear the "dl" structure where the current day lidar data will be put
  clear dl
  % Convert number to a string
  thedate = num2str(dates(id));
  if ~exist('dl');
    % Load the data
    infile=[ddir thedate '_chilbolton_halo-doppler-lidar.nc'];
    [dl,al] = load_nc_struct(infile);
    if isempty(dl)
      warning(['Doppler lidar data from ' thedate ' not found']);
      BL_TYPE(id,:) = 9999;  %if data is not present
      P_BL_7(id,:) = 9999;
      P_BL_8(id,:) = 9999;
      P_BL_1(id,:) = 9999;
      P_BL_2(id,:) = 9999;
      P_BL_3(id,:) = 9999;
      P_BL_4(id,:) = 9999; 
      P_BL_5(id,:) = 9999; 
      P_BL_6(id,:) = 9999; 
      P_BL_9(id,:) = 9999;  
      continue
    end
  end
  
  % Combine relevant day before and today lidar data into a single array
  % (use 30 mins each side for backscatter and 1 hour each side for v).
  if BL_TYPE(id,1) == 0
    it0 = 1;
    index_before_bk = find(d.time >= 24-0.5*dtime);   
    index_before_sk = find(d.time >= 24-0.5*dtime_sk);
    index_today_bk = find(dl.time < 23+0.5*dtime);   
    index_today_sk = find(dl.time < 23+0.5*dtime_sk);
    
    beta = cat(1,d.beta(index_before_bk,:),dl.beta(index_today_bk,:));
    time_bk = cat(1,d.time(index_before_bk)-24,dl.time(index_today_bk));
    v = cat(1,d.v(index_before_sk,:),dl.v(index_today_sk,:));
    time_sk = cat(1,d.time(index_before_sk)-24,dl.time(index_today_sk));
  else
    it0 = 2;
    index_today_bk = find(dl.time < 23+0.5*dtime);   
    index_today_sk = find(dl.time < 23+0.5*dtime_sk);
    
    beta = dl.beta(index_today_bk,:);
    time_bk = dl.time(index_today_bk);
    v = dl.v(index_today_sk,:);
    time_sk = dl.time(index_today_sk);
  end
  
  
     %%%%%%%%%%%%%%%%%%%%%%%%%%
     % Process the lidar data %
     %%%%%%%%%%%%%%%%%%%%%%%%%%
    
  % Remove the lowest three range gates as unreliable
  beta(:,1:3) = NaN;
  v(:,1:3) = NaN;
  
  % Set some variables and initialise the std and skewness arrays
  
  % Total number of range gates to use and the number to average over
  nr = 84;   % This number should be divisible by "sr"
  sr = 3;
  
  % Set the coordinates of the averaged time and range axes.  Typically the
  % data are centred on the hour, because that is the resolution of the
  % model data.
  arange = mean(reshape(dl.range(1:nr),sr,nr./sr));
  
  % Number of averaged time and range points
  atime = 0:23;
  ant = length(atime);
  anr = fix(nr./sr); % "fix" is equivalent to rounding down
  
  % Initialise the fields we wish to calculate on the averaged grid
  std_v = zeros(ant, anr);    % Raw standard deviation of velocity
  skew_v = zeros(ant, anr);   % Skewness
  
  for it = it0:24
    
    % Find the time indices for hour it (both 1 and 2 hour periods)
    use_bk = find(time_bk >= atime(it)-0.5*dtime & time_bk < atime(it)+0.5*dtime);
    use_sk = find(time_sk >= atime(it)-0.5*dtime_sk & time_sk < atime(it)+0.5*dtime_sk);
    
    if isempty(use_bk) || isempty(use_sk)
       hour = num2str(it);
       warning(['Doppler lidar data from hour ' hour ' not found']);
       BL_TYPE(id,it) = 9999;  %if data is not present
       P_BL_7(id,it) = 9999;
       P_BL_8(id,it) = 9999;
       P_BL_1(id,it) = 9999;
       P_BL_2(id,it) = 9999;
       P_BL_3(id,it) = 9999;
       P_BL_4(id,it) = 9999; 
       P_BL_5(id,it) = 9999; 
       P_BL_6(id,it) = 9999; 
       P_BL_9(id,it) = 9999;  
       continue 
    end 
    
    % Remove rays of data where all the velocities are "not a number",
    % NaN.
    good_ray_sk = ~isnan(nansum(v(use_sk,1:nr)',1)); 
    use_sk = use_sk(good_ray_sk);
    
    % Get the number of reliable rays in 2 hour period that want skewness
    % and turbulence averaged over
    good_st = length(use_sk);
    
    % v contains the full velocity field. v(use_sk,1:nr) contains
    % just the velocities within the height and time range of
    % interest. The following operation "reshapes" this to a matrix where
    % each group of 3 range gates is unfolded into 1, so each column of
    % the matrix corresponds to the "averaged" vertical grid    
    my_v = reshape(v(use_sk,1:nr), good_st.*sr,anr);     
    
    % Calculate the moments
    mean_v = nanmean(my_v);
    std_v(it,:) = nanstd(my_v);
    var_v = std_v(it,:).^2;
    skew_v(it,:) = nanmean((my_v-ones(good_st.*sr,1)*mean_v).^3)./std_v(it,:).^3;
    
    if fraction_good_data_reqd > 0
      % Find those averaged range pixels where fewer than the specified
      % fraction of high-resolution pixels are non-NaN, and set the
      % result to NaN, indicating that the data are unreliable here.
      levind = find(sum(isfinite(my_v))<good_st.*sr.*fraction_good_data_reqd);
      std_v(it,levind) = NaN;
      skew_v(it,levind) = NaN;
    end

    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Backscatter calculations %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    % Select subset of good beta in this hour
    beta_hr = beta(use_bk,:);
    % Find height of aerosol layer top
    % beta_log is array holding 0s and 1s marking where there is backscatter
    beta_log = isfinite(beta_hr);
    % Find the fraction of rays at each height with no backscatter
    frays = 100*(1-(sum(beta_log)/length(use_bk)));
    % Fix: ignore zero backscatter in gates 4 and 5 (these are sometimes
    % missing)
    frays(1:3) = 0;
    if frays(4) > 80
      frays(4) = 0;
    elseif frays(5) > 80
      frays(4:5) = 0;
    end 
    % Find the height of first aerosol free gate (>80% of backscatters
    % there are zero)
    aerosol(it) = dl.range(find(frays>80,1));
    
    % Determine the cloud base height
    ind = find(dl.range<=aerosol(it));
    ind_v = find(arange<=aerosol(it));
    range = ones(length(use_bk),1)*dl.range(ind)';
    
    % Remove all backscatter below threshold of back_thres
    beta_hr(find(beta_hr < back_thres)) = NaN;
    beta_cloudlog = isfinite(beta_hr);
    range(find(beta_cloudlog(:,ind) == 0)) = NaN;
    cl_min = min(range');
    cloud_base(it) = prctile(cl_min,5);
    cloud_base2(it) = NaN;
    cloud_base_ind = max(find(arange < cloud_base(it)));
    nrays_bk = length(use_bk);
    fclear(id,it) = sum(isnan(cl_min))/nrays_bk;
    phi_c(id,it) = fclear(id,it) < frac_clear_thres;
    
    % Calculate the bin edges needed for the histc function
    bin_edges = arange;
    % Calculate bin centres
    bin_centres = bin_edges + 54;

    two_lay = zeros(4,1);
    for j = 1:4
      if j == 1 
        window_index = 1:round(nrays_bk/3);
      elseif j == 2
        window_index = (round(nrays_bk/3)+1):round(2*nrays_bk/3);
      elseif j == 3
        window_index = round(2*nrays_bk/3):nrays_bk;
      else
        window_index = 1:round(nrays_bk);
      end

      % Count number in each bin and divide by the number of rays with a cloud
      binned_pdf = histc(cl_min(window_index),bin_edges);
      if sum(binned_pdf)>0
        binned_pdf = binned_pdf/sum(binned_pdf);
      
        % Find the location of the maximum value
        max_ind = find(binned_pdf == max(binned_pdf),1);
        
        % Search upwards until reach value<5% or end of array
        loc1 = max_ind+1;
        flag = 0;
        while loc1 <= max(ind_v) && flag == 0
          if binned_pdf(loc1) < 0.05 
            flag = 1;
          else
            loc1 = loc1+1;
          end
        end
        % Next check if there are any points above loc1 with binned_pdf>5%
        % and if there is, sum the rays in that region
        if flag == 1
          locs_above = loc1:max(ind_v);
          if max(binned_pdf(locs_above)) > 0.05
            two_lay(j) = 1;
            sum_above = sum(binned_pdf(locs_above));
            max_above_ind = locs_above(find(binned_pdf(locs_above) == max(binned_pdf(locs_above)),1));
            % Finally search back downwards and find first cloud free point
            loc11 = max_above_ind-1;
            flag2 = 0;
            while flag2 == 0
              if binned_pdf(loc11) < 0.05 
                flag2 = 1;
              else
                loc11 = loc11-1;
              end
            end
          else
            sum_above = 0;
          end
        else
          sum_above = 0;
        end
        
        % Search downwards until reach value<5% or start of array
        loc2 = max_ind-1;
        flag = 0;
        while loc2 >= 1 && flag == 0
          if binned_pdf(loc2) < 0.05 
            flag = 1;
          else
            loc2 = loc2-1;
          end
        end
        % Next check if there are any points below loc2 with binned_pdf>5%
        % and if there is, sum the rays in that region
        if flag == 1
          locs_below = 1:loc2;
          if max(binned_pdf(locs_below)) > 0.05
            two_lay(j) = 1;
            sum_below = sum(binned_pdf(locs_below));
            max_below_ind = locs_below(find(binned_pdf(locs_below) == max(binned_pdf(locs_below)),1));
            % Finally search back upwards and find first cloud free point
            if max_below_ind > 1
              loc22 = max_below_ind-1;
              flag2 = 0;
              while flag2 == 0
                if binned_pdf(loc22) < 0.05 
                  flag2 = 1;
                else
                  loc22 = loc22+1;
                end
              end
            else 
              sum_below = 0;
            end 
            
            else
              sum_below = 0;
            end
          else 
            sum_below = 0;
          end
        
        % Choose whether second layer is above or below the maximum for
        % the hrly data calculation
        if j == 4
          if two_lay(j) == 1
            if sum_above > sum_below
              gap_locs = loc1:loc11;
              cloud_base2(it) = bin_centres(loc11);
            else
              gap_locs = loc22:loc2;
              cloud_base2(it) = bin_centres(loc2);
            end
          end
        end
        
      end
    end

    phi_2lay(id,it) = sum(two_lay(1:3))/3;
    phi_2lay_hr(id,it) = two_lay(4);

    
    %%%%%%%%%%%%%%%%%%%
    % Calculate Pskew %
    %%%%%%%%%%%%%%%%%%%
    
    % If there's more than one layer of cloud and the surface layer is
    % stable then use gap_locs (i.e. the gap between the two thickest
    % clouds) to calculate skewness, otherwise just use the top third of
    % the aerosol layer.
    if phi_2lay_hr(id,it) == 1 && phi_st(id,it) > 0.5
     
      if length(gap_locs) == 1
         phi_skew_ind = gap_locs;
      else 
      phi_skew_ind = gap_locs(find(skew_v(it,gap_locs) == min(skew_v(it,gap_locs))));
      end 
      
      if isnan(phi_skew_ind) == 1
        phi_skew_ind = 1;
      end
    else
      nag = length(ind_v);
      top_third_ind = ind_v(round(2*nag/3):nag);
      if isfinite(nanmean(skew_v(it,top_third_ind))) == 1
        phi_skew_ind = top_third_ind(find(skew_v(it,top_third_ind) == min(skew_v(it,top_third_ind))));
        if isnan(phi_skew_ind) == 1
          phi_skew_ind = 1;
        end
      else
        if max(ind_v) >= find(isfinite(skew_v(it,:)) == 1)
          phi_skew_ind = max(ind_v(find(isfinite(skew_v(it,:)) == 1)));
        else 
          phi_skew_ind = 1;
        end
      end
      
    end
    
    
    % Find the sampling error in the skewness and variance measurements
    % at the range gate phi_skew_ind
    if isempty(phi_skew_ind)
    h = 1;
    else 
      h = phi_skew_ind;
    end 
    
    my_v_prime = my_v(:,h)-mean_v(h);
    my_v_prime = my_v_prime(find(isfinite(my_v_prime)));
    
    % Skewness calculation
    % Number of independent observations
    N_ind = indep_samples(my_v_prime.^3);
    % standard error of the sample skewness
    err_skew = sqrt(6/(N_ind));
    % probability of variance being less than 0
    z_skew = -skew_v(it,h)/err_skew;
    phi_skew(id,it) = 0.5*(1+erf(z_skew/sqrt(2))); 

    
    %%%%%%%%%%%%%%%%%%%
    % Calculate Pvar %
    %%%%%%%%%%%%%%%%%%%
    
    % If there's more than one layer of cloud and the surface layer is
    % stable then use gap_locs (i.e. the gap between the two thickest
    % clouds) to calculate variance, otherwise just use the top third of
    % the aerosol layer.
    if phi_2lay_hr(id,it) == 1 && phi_st(id,it) > 0.5
      
      if length(gap_locs) == 1
         phi_var_ind = gap_locs;
       else 
         phi_var_ind = gap_locs(find(var_v(gap_locs) == max(var_v(gap_locs))));
       end
       
      if isnan(phi_var_ind) == 1
        phi_var_ind = 1;
      end
    else
      nag = length(ind_v);
      top_third_ind = ind_v(round(2*nag/3):nag);
      phi_var_ind = top_third_ind(find(var_v(top_third_ind) == max(var_v(top_third_ind))));
      if isnan(phi_var_ind) == 1
        phi_var_ind = 1;
      end
    end 
    
   %phi_var_ind = phi_skew_ind;
    
    % Find the sampling error in the skewness and variance measurements
    % at the range gate phi_var_ind
   if isempty(phi_var_ind)
    h = 1;
    else 
      h = phi_var_ind;
    end 
   
    my_v_prime = my_v(:,h)-mean_v(h);
    my_v_prime = my_v_prime(find(isfinite(my_v_prime)));
      
    % Variance calculation
    % Number of independent observations
    N_ind = indep_samples(my_v_prime.^2);
    % standard error of the sample variance
    err_var = sqrt(2/(N_ind - 1))*var_v(h);
    % probability of variance being greater than var_thres
    z_var = (var_thres-var_v(h))/err_var;
    phi_var(id,it) = 1 - 0.5*(1+erf(z_var/sqrt(2)));

    
    %%%%%%%%%%%%%%%%%%%%%%
    % Decoupled decision %
    %%%%%%%%%%%%%%%%%%%%%%
    
    % If no cloud then set Pde=1 as it's not needed
    if phi_c(id,it) == 0
      phi_de(id,it) = 1;
    else
      
      % Create a subarray of var_v up to the cloud base and remove NaNs
      var_v_sub = [0 var_v(2:cloud_base_ind)];
      range_sub = [0 arange(2:cloud_base_ind)];
      var_v_sub = var_v_sub(isfinite(var_v_sub));
      range_sub = range_sub(isfinite(var_v_sub));
      
      % Find the range gate at just below half the aerosol depth
      bl_half = aerosol(it)/2;
      bl_half_ind = max(find(range_sub <= bl_half));
  
      % Check there's enough data to fit the quartic (else set Pde=0.5)
      if length(var_v_sub) > 4 && bl_half_ind > 2
      
        % Fit the quartic
        [p,s] = polyfit(range_sub,var_v_sub,4);
        f = polyval(p,range_sub);

        % Calculate error covariance matrix of p
        err_cov_p = (inv(s.R)*inv(s.R)').*(((s.normr)^2)/s.df);

        % Second derivative coefficients of variance fit 
        f_double_prime_coeff = [12*p(1) 6*p(2) 2*p(3)];

        % Values of second derivative 
        f_double_prime_val = polyval(f_double_prime_coeff,range_sub);

        % Find max of second derivative above the surface and below bl_half
        max_curve(id,it) = max(f_double_prime_val(2:bl_half_ind));
        % Find index of max second derivative
        max_curve_ind(id,it) = find(f_double_prime_val(1:bl_half_ind) == max_curve(id,it));

        % Calculate the error covariance matrix of f_double_prime_coeff
        Ct1 = [12.*range_sub(:).^2 6.*range_sub(:)];
        Ct2 = 2.*ones(length(range_sub(:)),1);
        Ct3 = 0 .*ones(length(range_sub(:)),1);
        Ct4 = 0 .*ones(length(range_sub(:)),1);
        Ct = (cat(2, Ct1,Ct2,Ct3,Ct4));
        for b = 1:length(range_sub)
          err_covar_f_double_prime = Ct(b,:)*err_cov_p*Ct(b,:)'; % Error covariance matrix of the second derivatives
          err_f_double_prime(b) = sqrt(diag(err_covar_f_double_prime)); % extract the diagonal - error at each range gate
        end 
        err_f_double_prime(max_curve_ind(id,it));

        % Calulate the probability of the maximum curvature being negative
        z_de = (-(max_curve(id,it))/err_f_double_prime(max_curve_ind(id,it)));
        phi_de(id,it) = 0.5*(1+erf(z_de/(sqrt(2))));

        % If the fit hasn't worked then just set Pde=0.5
        if isnan(phi_de(id,it))
          phi_de(id,it) = 0.5;
        end
      else
        phi_de(id,it) = 0.5;
      end
    end
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Define the boundary layer type probabilities %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    p_st = phi_st(id,it);
    p_var = phi_var(id,it);
    p_skew = phi_skew(id,it);
    p_c = phi_c(id,it);
    p_de = phi_de(id,it);
    p_2lay = phi_2lay(id,it);
      
    % If aerosol layer < 200m then don't diagnose
    if isnan(p_st) == 1
      P_BL_1(id,it) = 8888;
      P_BL_2(id,it) = 8888;
      P_BL_3(id,it) = 8888;
      P_BL_4(id,it) = 8888;
      P_BL_5(id,it) = 8888;
      P_BL_6(id,it) = 8888; 
      P_BL_7(id,it) = 8888; 
      P_BL_8(id,it) = 8888; 
      P_BL_9(id,it) = 8888;
    elseif aerosol(it) < 200
      P_BL_1(id,it) = 2222;
      P_BL_2(id,it) = 2222;
      P_BL_3(id,it) = 2222;
      P_BL_4(id,it) = 2222;
      P_BL_5(id,it) = 2222;
      P_BL_6(id,it) = 2222; 
      P_BL_7(id,it) = 2222; 
      P_BL_8(id,it) = 2222; 
      P_BL_9(id,it) = 2222;
    % If cloud in first range gate then assume well-mixed
    elseif cloud_base(it) < 270
      P_BL_1(id,it) = p_st;
      P_BL_2(id,it) = 0; 
      P_BL_3(id,it) = 1-p_st;
      P_BL_4(id,it) = 0;
      P_BL_5(id,it) = 0;
      P_BL_6(id,it) = 0;
      P_BL_7(id,it) = 0;
      P_BL_8(id,it) = 0;
      P_BL_9(id,it) = 0;
    % Otherwise use normal formulae
    else      
      P_BL_1(id,it) = p_c*p_st*(1-p_skew*p_var);
      P_BL_2(id,it) = p_c*p_st*p_skew*p_var*(1-p_2lay); 
      P_BL_3(id,it) = p_c*(1-p_st)*p_skew*(1-p_var*(1-p_de));
      P_BL_4(id,it) = p_c*(1-p_st)*p_skew*p_var*(1-p_de);
      P_BL_5(id,it) = p_c*(1-p_st)*(1-p_skew)*p_2lay;
      P_BL_6(id,it) = p_c*(1-p_st)*(1-p_skew)*(1-p_2lay);
      P_BL_7(id,it) = (1-p_c)*p_st;
      P_BL_8(id,it) = (1-p_c)*(1-p_st);
      P_BL_9(id,it) = p_c*p_st*p_skew*p_var*p_2lay;
    end
    
% End the sonic check loops
  end
end
end
end


for id=1:length(dates)
  for it=1:24
    
    if isfinite(sonic_ind(id)) == 0
      P_BL_1(id,it) = 8888;
      P_BL_2(id,it) = 8888;
      P_BL_3(id,it) = 8888;
      P_BL_4(id,it) = 8888;
      P_BL_5(id,it) = 8888;
      P_BL_6(id,it) = 8888; 
      P_BL_7(id,it) = 8888; 
      P_BL_8(id,it) = 8888; 
      P_BL_9(id,it) = 8888; 
    else
      sonic_good = isfinite(nansum(phi_st_full(sonic_ind(id),:)));
      if sonic_good == 0
        P_BL_1(id,it) = 8888;
        P_BL_2(id,it) = 8888;
        P_BL_3(id,it) = 8888;
        P_BL_4(id,it) = 8888;
        P_BL_5(id,it) = 8888;
        P_BL_6(id,it) = 8888; 
        P_BL_7(id,it) = 8888; 
        P_BL_8(id,it) = 8888; 
        P_BL_9(id,it) = 8888;
      end
    end        

    % Find the most probable type
    P_BL = cat(1, P_BL_1(id,it), P_BL_2(id,it),P_BL_3(id,it),P_BL_4(id,it),P_BL_5(id,it),P_BL_6(id,it), P_BL_7(id,it),P_BL_8(id,it),P_BL_9(id,it));
    P_BL_sum(id,it) = nansum(P_BL);
    % If sum close to one then ok
    if abs(P_BL_sum(id,it)-1) < 0.002
      BL_TYPE(id,it) = min(find(P_BL == max(P_BL)));
    % If instead there's a error in the data files then use that
    elseif P_BL_1(id,it) == 9999 || P_BL_1(id,it) == 8888 || P_BL_1(id,it) == 2222
        BL_TYPE(id,it) = P_BL_1(id,it);
    % Else one of the p's is NaN
    else 
      BL_TYPE(id,it) = NaN;
    end 
    
  end
end

% Output data matrix to a file

% Re-assign BL_TYPE numbers to match what is in the output file (see file header below)

for id = 1:length(dates)
    for it = 1:24
        if BL_TYPE(id,it) == 1 
        BL_TYPE_OUT(id,it) = 2;
        elseif BL_TYPE(id,it) == 2
        BL_TYPE_OUT(id,it) = 4;
        elseif BL_TYPE(id,it) == 3
        BL_TYPE_OUT(id,it) = 6;
        elseif BL_TYPE(id,it) == 4
        BL_TYPE_OUT(id,it) = 7;
        elseif BL_TYPE(id,it) == 5
        BL_TYPE_OUT(id,it) = 8;
        elseif BL_TYPE(id,it) == 6
        BL_TYPE_OUT(id,it) = 9;
        elseif BL_TYPE(id,it) == 7
        BL_TYPE_OUT(id,it) = 1;
        elseif BL_TYPE(id,it) == 8
        BL_TYPE_OUT(id,it) = 5;
        elseif BL_TYPE(id,it) == 9
        BL_TYPE_OUT(id,it) = 3;
end
end 
end  


for id=1:length(dates)
  fid = fopen([odir 'P_BL_' num2str(dates(id)) '.txt'],'wt');
  fprintf(fid,'%s\n','% Output from bl_type.m')
  fprintf(fid,'%s\n',['% Date: ' num2str(dates(id))])
  fprintf(fid,'%s\n','% Each row of this file contains 11 numbers, one for each hour of the day (centred on the hour):')
  fprintf(fid,'%s\n','% 1. Hour UTC')
  fprintf(fid,'%s\n','% 2. Most probable boundary-layer type from 1 to 9')
  fprintf(fid,'%s\n','% 3. Probability of type 1 (Ia: Stable boundary layer, no cloud)')
  fprintf(fid,'%s\n','% 4. Probability of type 2 (Ib: Stratus-topped stable boundary layer, no cumulus)')
  fprintf(fid,'%s\n','% 5. Probability of type 3 (Ic: Forced cumulus in a stable boundary layer with decoupled stratocumulus above)')
  fprintf(fid,'%s\n','% 6. Probability of type 4 (II: Decoupled stratocumulus over a stable surface layer, no cumulus)')
  fprintf(fid,'%s\n','% 7. Probability of type 5 (IIIa: Single mixed layer, no cloud)')
  fprintf(fid,'%s\n','% 8. Probability of type 6 (IIIb: Single stratocumulus-topped mixed layer, no cumulus)')
  fprintf(fid,'%s\n','% 9. Probability of type 7 (IV: Decoupled stratocumulus over an unstable surface layer)')
  fprintf(fid,'%s\n','% 10. Probability of type 8 (V: Decoupled stratocumulus over cumulus)')
  fprintf(fid,'%s\n','% 11. Probability of type 9 (VI: Cumulus-capped layer)') 
  temp(:,1) = 0:23;
  temp(:,2) = BL_TYPE_OUT(id,:)';
  temp(:,3) = P_BL_7(id,:)';
  temp(:,4) = P_BL_1(id,:)';
  temp(:,5) = P_BL_9(id,:)';
  temp(:,6) = P_BL_2(id,:)';
  temp(:,7) = P_BL_8(id,:)';
  temp(:,8) = P_BL_3(id,:)';
  temp(:,9) = P_BL_4(id,:)';
  temp(:,10) = P_BL_5(id,:)';
  temp(:,11) = P_BL_6(id,:)';
  fprintf(fid,'%1g\t %1g\t %4.3f\t %4.3f\t %4.3f\t %4.3f\t %4.3f\t %4.3f\t %4.3f\t %4.3f\t %4.3f\n', temp');
  fclose(fid)
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Addition plotting code (plots the final day only) %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  % Plot the results
  fontsize = 13;
  
  % First figure: backscatter and vertical velocity standard deviation
  figure('Position',[50 50 700 350])
  set(gcf,'DefaultAxesFontSize',fontsize);
  set(gcf,'DefaultTextFontSize',fontsize);
  set(gcf,'DefaultAxesLayer','Top');
  clf

  % Plot 0-24 hours and 0-2.5 km
  theaxis = [0 24 0 2.5];
  yplotpos = theaxis(4)-0.5;
  
  % Only plot every 2nd point to keep the plotted files small
  res = 2;

  % First panel: raw lidar backscatter using a logarithmic scale
  subplot(2,1,1);
  pcolor(time_bk(1:res:end),dl.range./1000,log10(beta(1:res:end,:))')
  shading flat
  caxis([-6.5 -4]);
  axis(theaxis);
  ylabel('Height (km)');
  set(gca,'xtick',[0:3:24],'ytick',[0:1:2]);
  text(0,yplotpos.*0.9,'  \bf Lidar backscatter (log_{10}[m^{-1} sr^{-1}])');
  % Overplot the boundary-layer height estimate
  hold on
  plot(0:1:23,aerosol./1000,'k','linewidth',3);
  plot(0:1:23,cloud_base./1000,'k--','linewidth',3);
  plot(0:1:23,cloud_base2./1000,'kx--','linewidth',3);
  text(-3,2.7,'(a)');

  bl_pos = 2.7;
  for i = 1:24
      if BL_TYPE(id,i) == 7
        text(i-1,bl_pos,'Ia','FontSize',11,'FontWeight','bold','HorizontalAlignment','center');
      elseif BL_TYPE(id,i) == 1
        text(i-1,bl_pos,'Ib','FontSize',11,'FontWeight','bold','HorizontalAlignment','center');  
      elseif BL_TYPE(id,i) == 2
        text(i-1,bl_pos,'II','FontSize',11,'FontWeight','bold','HorizontalAlignment','center'); 
      elseif BL_TYPE(id,i) == 8
        text(i-1,bl_pos,'IIIa','FontSize',11,'FontWeight','bold','HorizontalAlignment','center'); 
      elseif BL_TYPE(id,i) == 3
        text(i-1,bl_pos,'IIIb','FontSize',11,'FontWeight','bold','HorizontalAlignment','center'); 
      elseif BL_TYPE(id,i) == 4
        text(i-1,bl_pos,'IV','FontSize',11,'FontWeight','bold','HorizontalAlignment','center'); 
      elseif BL_TYPE(id,i) == 5
        text(i-1,bl_pos,'V','FontSize',11,'FontWeight','bold','HorizontalAlignment','center'); 
      elseif BL_TYPE(id,i) == 6
        text(i-1,bl_pos,'VI','FontSize',11,'FontWeight','bold','HorizontalAlignment','center');
      elseif BL_TYPE(id,i) == 9
        text(i-1,bl_pos,'Ic','FontSize',11,'FontWeight','bold','HorizontalAlignment','center');
      end
  end
  mycolorbar
  
  % Second panel: vertical velocity standard deviation
  subplot(2,1,2);
  contourf(atime,arange./1000,std_v',[0:0.05:2]);shading flat
  hold on
  % Overplot the boundary-layer height estimate
  plot(0:1:23,aerosol./1000,'k','linewidth',3);
  plot(0:1:23,cloud_base./1000,'k--','linewidth',3);
  plot(0:1:23,cloud_base2./1000,'k--','linewidth',3);
  axis(theaxis);
  caxis([0 1]);
  ylabel('Height (km)');
  xlabel('Time (UTC)');
  set(gca,'xtick',[0:3:24],'ytick',[0:1:2]);
  text(0,yplotpos,'  \bf Vertical velocity standard deviation (m s^{-1})');
  text(-3,2.7,'(b)');
  mycolorbar;
  chiljet
  
  % Second figure: skewness and H_hr values
  figure('Position',[50 50 700 350])
  set(gcf,'DefaultAxesFontSize',fontsize);
  set(gcf,'DefaultTextFontSize',fontsize);
  set(gcf,'DefaultAxesLayer','Top');
  clf
  
  % First panel: skewness
  subplot(2,1,1);
  contourf(atime, arange./1000, skew_v',[-10 -1:0.2:1]);shading flat
  hold on
  % Overplot the boundary-layer height estimate
  plot(0:1:23,aerosol./1000,'k','linewidth',3);
  plot(0:1:23,cloud_base./1000,'k--','linewidth',3);
  plot(0:1:23,cloud_base2./1000,'k--','linewidth',3);
  axis(theaxis);
  caxis([-1 1]);
  ylabel('Height (km)');
  %xlabel('Time (UTC)');
  set(gca,'xtick',[0:3:24],'ytick',[0:1:2]);
  text(0,yplotpos,'  \bf Vertical velocity skewness');
  text(-3,2.7,'(c)');
  mycolorbar
  litmus
  
  % Second panel: H_hr and H_err  
  H_hr = H_full(find(H_dates == dates(id)),:);
  H_err = H_err_full(find(H_dates == dates(id)),:);
  
  subplot(2,1,2);
  errorbar(0:1:23, H_hr, H_err)
  ylabel('H (Wm^-^2)')
  xlabel('Time (UTC)')
  set(gca,'xtick',[0:3:24]);
  grid
  axis(theaxis);
  ylim([ -50 400]);
  %xlim([ 0 24])
  text(-3,432,'(d)');
  mycolorbar
