%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% pst.m                                                                  %
%                                                                        %
% This program reads in data from the sonic anemometer for chosen array  %
% of dates. Using the T and w timeseries H is calculated for each of data%   
% centred on the hour. Note this requires the sonic data from the        %
% previous day to calculate H for 00Z. When H is found the error on H is %
% found using an autocorrelation technique to determine the number of    %
% independent samples in the data series for the chosen hour. This is    %
% done using a function called indep_samples. The number of independent  %
% samples can then be used to calculate the standard error in H. The     %
% standard error is then used to determine the probability that the      %
% sensible heat flux is less than 0 (phi). The array of phi values is    %
% then saved as a text file. The name of the text file is set at the top %
% of the code. Also exported is an array showing when the sonic data is  %
% missing (missing_sonic).                                               %
%                                                                        %
% Copyright (c) 2012-2013 Natalie Harvey and the University of Reading   %
%                                                                        %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Specify dates
date0 = 20110730;  % This is the day before the first element of dates
dates = [20110731,20110801:20110803];

% Specify temporal resolution for calculating H (in hours)
dtime = 1;

% Specify sonic data location
ddir = ['./data/'];

% Specify directory for output
odir = ['./pst/'];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Define output file names
outfile1=[odir 'pst_' num2str(dates(1)) '_' num2str(dates(length(dates))) '.txt'];
outfile2=[odir 'H_' num2str(dates(1)) '_' num2str(dates(length(dates))) '.txt'];
outfile3=[odir 'H_err_' num2str(dates(1)) '_' num2str(dates(length(dates))) '.txt'];

% Initialise phi
phi = zeros(length(dates),24);
H_hr = zeros(length(dates),24);
H_err = zeros(length(dates),24);

% Loop over dates
for id = 1:length(dates);

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Read in the sonic data and combine into single-day periods  %
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  %%%%%%%%%%%%%%
  % Day before %
  %%%%%%%%%%%%%%
  
  % Convert previous day date to string
  if id == 1
    day_before = num2str(date0);
  else
    day_before = num2str(dates(id-1));
  end

  % Load sonic data from previous day
  clear e
  infile=[ddir 'cfarr_sonic_' day_before '.nc'];
  [e,a]=load_nc_struct(infile);
  if isempty(e)
    warning(['Sonic Anemometer data from ' day_before ' not found']);
    phi(id,1) = NaN;
    H_hr(id,1) = NaN;
    H_err(id,1) = NaN;
  end
 
  %%%%%%%%%
  % Today %
  %%%%%%%%%
  
  % Convert date to string
  thedate = num2str(dates(id));

  % Load sonic data from today
  clear el
  infile=[ddir 'cfarr_sonic_' thedate '.nc'];
  [el,bl]=load_nc_struct(infile);
  if isempty(el)
    warning(['Sonic Anemometer data from ' thedate ' not found']);
    phi(id,:) = NaN;
    H_hr(id,:) = NaN;
    H_err(id,:) = NaN;
    continue
  end
 
  % Combine relevant day before and today sonic data into a single array
  % (use 30 mins each side for all data). Also converts time values to
  % hour units
  if isfinite(phi(id,1)) == 1
    it0 = 1;
    index_before = find(e.time./3600 >= 24-0.5*dtime);
    index_today = find(el.time./3600 < 23+0.5*dtime);
    w = cat(1,e.w(index_before),el.w(index_today));
    T = cat(1,e.temperature(index_before),el.temperature(index_today));
    time_sonic = cat(1,e.time(index_before)./3600-24,el.time(index_today)./3600);
  else
    it0 = 2;
    index_today = find(el.time./3600 < 23+0.5*dtime);
    w = el.w(index_today);
    T = el.temperature(index_today);
    time_sonic = el.time(index_today)./3600;
  end
    
  % Loop over the hours
  atime = 0:23;
  for it=it0:24
  
     %%%%%%%%%%%%%%%%%%%%%%%%%%
     % Process the sonic data %
     %%%%%%%%%%%%%%%%%%%%%%%%%%
  
     % Constants needed to calculate H 
     rho=1.2; % density of air
     Cp=1004; % specific heat capacity of air at constant pressure

     % Find the time indices for hour it
     use_H = find(time_sonic >= atime(it)-0.5*dtime & time_sonic < atime(it)+0.5*dtime);
     
     % Find the fraction of missing data in hour it
     total_no = length(use_H);              % total number of data points
     use_H = use_H(find(isfinite(w(use_H)) & isfinite(T(use_H))));
     no_OK = length(use_H);             % amount of good data points
     missing_sonic = 1 - (no_OK/total_no);  % fraction of missing data
 
     hour = num2str(it);
     if missing_sonic >= 0.9 || total_no == 0
       warning(['Too much missing data for hour ' hour ]);
       phi(id,it) = NaN;
       H_hr(id,it) = NaN;
       H_err(id,it) = NaN;
       continue
     end
     
    % Calculate the standard deviation and mean of the sonic w and T
    std_w = nanstd(w(use_H));
    std_T = nanstd(T(use_H));
    mean_T = nanmean(T(use_H));
    mean_w = nanmean(w(use_H));
      
    % Calculate w' and T'
    wprime = w(use_H) - mean_w;
    Tprime = detrend(T(use_H),'linear'); % detrend function removes mean
                                                    % and a linear fit
    % Calculate H and its hour mean
    H = rho.*Cp.*(Tprime.*wprime);
    H_hr(id,it) = nanmean(H);

    % Use autocorrelation method to determine the sampling error on H for
    % each hour
    N_ind = indep_samples_fft(H);
    
    % Calculate the error in H due to sampling
    err_H(id,it) = nanstd(H)/sqrt(N_ind);
    
    % Calculate the probability that the sensible heat flux is less than 0
    z = -H_hr(id,it)/err_H(id,it);
    phi(id,it) = 0.5*(1+erf(z/sqrt(2)));
    
  end
end 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Export phi, H_hr and H_hr_err arrays %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Export the dates and data values
% One row per day, each containing the date in column 1 followed by 24 data values

phi_export = cat(2, dates', phi);
H_export =  cat(2, dates', H_hr);
H_err_export =  cat(2, dates', err_H);

fid = fopen(outfile1,'wt');
fprintf(fid,'%d\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\n ',phi_export');
fclose(fid);

fid = fopen(outfile2,'wt');
fprintf(fid,'%d\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\n ',H_export');
fclose(fid);

fid = fopen(outfile3,'wt');
fprintf(fid,'%d\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\t %1g\n ',H_err_export');
fclose(fid);
